#version 330

layout (points) in;
layout (triangle_strip) out;
layout (max_vertices = 4) out;

uniform mat4 uni_ViewProjectionMatrix;
uniform vec3 uni_CameraPos;

out vec2 var_TexCoord;

// The geometry Shader takes the positions and creates a billboard
// consisting of 4 vertices (transformed right into clip space) out of it
void main(void)
{
	float scale = 100.0;

	//the position from vertex shader
	vec3 Pos = gl_in[0].gl_Position.xyz;
	
	//camera vectors
	vec3 toCamera = normalize(uni_CameraPos - Pos);
    vec3 up = vec3(0.0, 1.0, 0.0);
    vec3 right = cross(toCamera, up);
	
	//now create the billboard by creating 4 vertices
	Pos -= (right * scale * 0.5);
    gl_Position = uni_ViewProjectionMatrix * vec4(Pos, 1.0);
    var_TexCoord = vec2(0.0, 0.0);
    EmitVertex();

    Pos.y += 1.0 * scale;
    gl_Position = uni_ViewProjectionMatrix * vec4(Pos, 1.0);
    var_TexCoord = vec2(0.0, 1.0);
    EmitVertex();

    Pos.y -= 1.0 * scale;
    Pos += right * scale;
    gl_Position = uni_ViewProjectionMatrix * vec4(Pos, 1.0);
    var_TexCoord = vec2(1.0, 0.0);
    EmitVertex();

    Pos.y += 1.0 * scale;
    gl_Position = uni_ViewProjectionMatrix * vec4(Pos, 1.0);
    var_TexCoord = vec2(1.0, 1.0);
    EmitVertex();

    EndPrimitive();
}