#version 330

uniform sampler2D uni_RTT_Texture;
uniform sampler2D uni_LightScatterTexture;
uniform vec2 uni_LightPosScreenSpace;

//uniform float uni_LightExposure = 0.01;
//uniform float uni_LightDecay = 0.9;
//uniform float uni_LightDensity = 0.9;
//uniform float uni_LightWeight = 9.65;
uniform float uni_LightExposure = 0.0084;
uniform float uni_LightDecay = 1.0;
uniform float uni_LightDensity = 0.84;
uniform float uni_LightWeight = 5.65;

in vec2 var_TexCoord;

out lowp vec4 frag_Color;

const int NUM_SAMPLES = 40;

vec2 poissonDisk[16] = vec2[](
vec2( -0.94201624, -0.39906216 ),
vec2( 0.94558609, -0.76890725 ),
vec2( -0.094184101, -0.92938870 ),
vec2( 0.34495938, 0.29387760 ),
vec2( -0.91588581, 0.45771432 ),
vec2( -0.81544232, -0.87912464 ),
vec2( -0.38277543, 0.27676845 ),
vec2( 0.97484398, 0.75648379 ),
vec2( 0.44323325, -0.97511554 ),
vec2( 0.53742981, -0.47373420 ),
vec2( -0.26496911, -0.41893023 ),
vec2( 0.79197514, 0.19090188 ),
vec2( -0.24188840, 0.99706507 ),
vec2( -0.81409955, 0.91437590 ),
vec2( 0.19984126, 0.78641367 ),
vec2( 0.14383161, -0.14100790 )
);

float randomNumber(vec4 seed4)
{
	float dot_product = dot(seed4, vec4(12.9898,78.233,45.164,94.673));
    return fract(sin(dot_product) * 43758.5453);
}

vec3 scatterValue()
{
	vec2 texCoord = var_TexCoord.st;
	
	float dist = distance(texCoord,uni_LightPosScreenSpace);
	vec2 deltaTextCoord = vec2( texCoord - uni_LightPosScreenSpace.xy );
	deltaTextCoord *= 1.0 / (float(NUM_SAMPLES)) * uni_LightDensity;
	
	vec3 scatter = texture(uni_LightScatterTexture, texCoord ).rgb;
	
	float illuminationDecay = 1.0;
	for(int i=0; i < NUM_SAMPLES ; i++)
	{
		//poisson sampling is awesome for a lower number of samples
		//but the index calculation is pretty heavyweight if using higher loopcounts!
		
		texCoord -= deltaTextCoord;
		//int index = int(16.0*randomNumber(vec4(gl_FragCoord.xyy, i)))%16;
		//lowp vec3 sample = texture(uni_LightScatterTexture, texCoord + poissonDisk[index] / 700.0 ).rgb;
		lowp vec3 sample = texture(uni_LightScatterTexture, texCoord).rgb;
		
        sample *= illuminationDecay * uni_LightWeight;
        scatter += sample;
        illuminationDecay *= uni_LightDecay;
	}
	return scatter;
}

void main(void)
{
	lowp vec3 color = texture(uni_RTT_Texture, var_TexCoord).rgb;
	
	frag_Color=vec4(color + scatterValue() * uni_LightExposure,1.0);
	//if(distance(var_TexCoord,uni_LightPosScreenSpace)<0.1)
		//frag_Color.r = 0.0;
	//frag_Color=vec4(uni_LightPosScreenSpace,0.0,1.0);
}