#version 330

uniform sampler2D uni_RTT_Texture;
uniform sampler2D uni_RTT_DepthTexture;
uniform vec3 uni_CameraViewDir;
uniform vec3 uni_CameraPos;
uniform float uni_Time;

in vec2 var_TexCoord;
in vec3 var_ViewVec;

out vec4 frag_Color;


const mat4 colorMatrix = mat4(
   0.30, 0.0, 0.0, 0.0, // first column 
   0.0, 0.6, 0.2, 0.0, // second column
   0.3, 0.4, 0.8, 0.0, // third column
   0.0, 0.0, 0.0, 1.0 // fourth column
);


const float amplitudes[4] = float[4](0.0045, 0.0015, 0.003, 0.0008);
const float frequencies[4] = float[4](7.3, -17.4, 14.8, -33.12);
const float phases[4] = float[4](0.0, 0.5, 0.3, 0.2);

uniform float zFar = 10000.0f;
uniform float zNear = 1.0f;

void main(void)
{
	//texcoords are from -1 to 1
	//take a smaller area than the full screen
	vec2 newCoord = var_TexCoord*0.45 + vec2(0.5,0.5);
	
	// **** WAVY LOOKUP
	float offsetAmount = newCoord.s + uni_Time;
	for(int i = 0;i<1;++i)
	{
		float offset = amplitudes[i] * sin(frequencies[i] * offsetAmount + phases[i]);
		newCoord.t += offset;
		newCoord.s -= offset * 0.5;
	}
	
	// **** POSITION RECONSTRUCTION FROM DEPTH TEXTURE
	vec3 u = normalize(var_ViewVec);
	vec3 v = normalize(uni_CameraViewDir);
	
	highp float depthBufferValue = texture(uni_RTT_DepthTexture,newCoord).r;
	
	highp float zn = 2.0 * depthBufferValue - 1.0; //scale [-1 1]
	highp float zEye =  2.0 * zNear * zFar / (zFar + zNear - zn * (zFar - zNear));
	
	//this accounts for the projection
	float bla = dot(u,v);
	float dist = zEye / bla;
	
	vec3 worldPos = (dist) * u + uni_CameraPos;
	
	
	// **** GET COLOR AND DO COLOR CORRECTION
	vec4 texColor = vec4(texture(uni_RTT_Texture, newCoord).rgb,1.0);
	
	float depthFactor = smoothstep(0.0,0.9,uni_CameraPos.y / 200);
	vec3 fogColor = vec3(0.1,0.6,0.9) * depthFactor;
	float fogFactor = 0.003; //smoothstep(0.0001, 0.0005, uni_CameraPos.y / 200000);//0.0005
	float farFog = min(exp(- fogFactor * (dist-500)),1); //dense fog starting at distance 500
	float nearFog = min(exp(- 0.0009 * (dist-50)),1); //below 50 is no fog
	
	vec4 foggedColor = vec4( mix(texColor.rgb,fogColor,1-(farFog*nearFog)), texColor.a);
	vec4 transformedColor = colorMatrix * foggedColor;

	//frag_Color =vec4( vec3(fog), 1.0);
	//frag_Color = foggedColor;
	frag_Color = transformedColor;
	gl_FragDepth = depthBufferValue;
	//frag_Color = vec4(mix(transformedColor,vec3(worldPos)*0.001,0.5),1.0);
}