#version 330

layout(points) in;
layout(points,max_vertices = 2) out;

in vec3 var_Pos[];
in float var_Scale[];
in float var_Rotation[];
flat in int var_PType[];
in vec3 var_PVelocity[];
in float var_PLifetime[];

out vec3 out_Position;
out float out_Rotation;
out float out_Scale;
flat out int out_Type;
out vec3 out_Velocity;
out float out_Lifetime;

uniform float uni_Time;
uniform float uni_TimeDelta;
uniform sampler1D uni_RandomTexture;

#define PTYPE_LAUNCHER 0
#define PTYPE_SIMPLE 1

//random vector with values from -1 to 1
vec3 getRandomVector()
{
	return texture(uni_RandomTexture,fract(uni_Time)).xyz;
}

//random value scaled to 0-1
float getRandomValue(in float offset)
{
	return (texture(uni_RandomTexture,fract(uni_Time)+offset).x + 1.0) / 2.0;
}

vec3 getRandomVectorWithOffset(in float offset)
{	
	return texture(uni_RandomTexture,fract(uni_Time)+offset).xyz;
}

void main()
{
	float age = var_PLifetime[0] + uni_TimeDelta;
	
	if(var_PType[0] == PTYPE_LAUNCHER)
	{
		if(age >= 0.5)
		{
			//spawn 1 bubble
			for(int i=0;i<1;++i)
			{
				vec3 direction = vec3(0.0f, 1.0f, 0.0f);

				//create a particle
				out_Type = PTYPE_SIMPLE;
				vec3 offset = getRandomVectorWithOffset(i/15.0);

				out_Position = var_Pos[0] + vec3(offset.x,0.0f,offset.z) * 400.0f; //begin at launcher location
				out_Rotation = 0.0f;
				out_Scale = getRandomValue(i/15.0+(1/20))*10.0;
				out_Velocity = normalize(direction) * (10.0 + 40 * getRandomValue(i/15.0+0.123));
				out_Lifetime = 0.0;
				EmitVertex();
				EndPrimitive();
			}
			
			//reset age
			age = 0.0;
		}
		//output launcher again
		out_Type = PTYPE_LAUNCHER;
		out_Position = var_Pos[0]; //launcher always stays here
		out_Rotation = 0.0;
		out_Scale = 0.0;
		out_Velocity = var_PVelocity[0]; //currently unused
		out_Lifetime = age;
		EmitVertex();
		EndPrimitive();
	}
	else if (var_PType[0] == PTYPE_SIMPLE)
	{
		//only output if particle still alive
		if(var_Pos[0].y < 190.0f)
		{
			vec3 deltaPos = uni_TimeDelta * var_PVelocity[0];
			//vec3 slowDown = 30 * vec3(0.0,-9.81,0.0) * uni_TimeDelta;
	
			out_Type = PTYPE_SIMPLE;
			out_Position = var_Pos[0] + deltaPos;
			out_Rotation = var_Rotation[0];
			out_Scale = var_Scale[0];
			out_Velocity = var_PVelocity[0];
			out_Lifetime = age;
			EmitVertex();
			EndPrimitive();
		}
	}
}