#version 330

uniform sampler2D uni_DiffuseMap;
uniform vec3 uni_CameraPos;

in vec2 var_TexCoord;
flat in vec3 var_EyeVec;

out vec4 frag_Color;
out vec4 frag_LSOcclusion;



const mat4 colorMatrix = mat4(
   0.30, 0.0, 0.0, 0.0, // first column 
   0.0, 0.6, 0.2, 0.0, // second column
   0.3, 0.4, 0.8, 0.0, // third column
   0.0, 0.0, 0.0, 1.0 // fourth column
);

//simple texture shader, supports blending
void main(void)
{
	//get color from texture
	vec4 texCol = texture(uni_DiffuseMap, var_TexCoord);
	
	float dist = length(var_EyeVec);

	float depthFactor = smoothstep(0.0,0.9,uni_CameraPos.y / 200);
	vec3 fogColor = vec3(0.1,0.6,0.9) * depthFactor;
	float fogFactor = 0.003; //smoothstep(0.0001, 0.0005, uni_CameraPos.y / 200000);//0.0005
	float farFog = min(exp(- fogFactor * (dist-500)),1); //dense fog starting at distance 500
	float nearFog = min(exp(- 0.0009 * (dist-50)),1); //below 50 is no fog
	vec4 foggedColor = vec4( mix(texCol.rgb,fogColor,1-(farFog*nearFog)), texCol.a);
	foggedColor.a *= farFog;
	
	frag_Color = colorMatrix * foggedColor;
    frag_LSOcclusion = vec4(0,0,0,1.0);
}