#version 330

layout(points) in;
layout(points,max_vertices = 26) out;

in vec3 var_Pos[];
in float var_Scale[];
in float var_Rotation[];
flat in int var_PType[];
in vec3 var_PVelocity[];
in float var_PLifetime[];

out vec3 out_Position;
out float out_Rotation;
out float out_Scale;
flat out int out_Type;
out vec3 out_Velocity;
out float out_Lifetime;

uniform float uni_Time;
uniform float uni_TimeDelta;
uniform sampler1D uni_RandomTexture;

#define PTYPE_LAUNCHER 0
#define PTYPE_SIMPLE 1
#define PTYPE_SMOKE 2

//random vector with values from -1 to 1
vec3 getRandomVector()
{
	return texture(uni_RandomTexture,fract(uni_Time)).xyz;
}

//random value scaled to 0-1
float getRandomValue(in float offset)
{
	return (texture(uni_RandomTexture,fract(uni_Time)+offset).x + 1.0) / 2.0;
}

vec3 getRandomVectorWithOffset(in float offset)
{	
	return texture(uni_RandomTexture,fract(uni_Time)+offset).xyz;
}

void main()
{
	float age = var_PLifetime[0] + uni_TimeDelta;
	
	if(var_PType[0] == PTYPE_LAUNCHER)
	{
		if(age >= 0.01)
		{
			//spawn 25 particles - we use 2 emitters to keep the max_vertices count lower
			for(int i=0;i<25;++i)
			{
			vec3 direction = getRandomVectorWithOffset(i/15.0);
			direction.y = max(direction.y,0.5);
			vec3 otherRandom = getRandomVectorWithOffset(i/15.0+(1/20));
			
			//create a particle
			out_Type = PTYPE_SIMPLE;
			out_Position = var_Pos[0]; //begin at launcher location
			out_Rotation = 1.0; //rotation is used for alpha here
			out_Scale = 50.0;
			out_Velocity = normalize(direction) * (300.0 + 500 * getRandomValue(i/15.0+0.123));
			out_Lifetime = 0.0;
			EmitVertex();
			EndPrimitive();
			}
			
			//reset age
			age = 0.0;
		}
		//output launcher again
		out_Type = PTYPE_LAUNCHER;
		out_Position = var_Pos[0]; //launcher always stays here
		out_Rotation = 1.0; //rotation is alpha here
		out_Scale = 0.0;
		out_Velocity = var_PVelocity[0]; //currently unused
		out_Lifetime = age;
		EmitVertex();
		EndPrimitive();
	}
	else if (var_PType[0] == PTYPE_SIMPLE)
	{
		if(var_Pos[0].y <= 200.0f)
		{
			//convert to smoke
			out_Type = PTYPE_SMOKE;
			out_Position = var_Pos[0];
			out_Rotation = var_Rotation[0];
			out_Scale = var_Scale[0];
			out_Velocity = vec3(0,40,0);
			out_Lifetime = 0.0;
			EmitVertex();
			EndPrimitive();
		}
		else if(age < 3.0) //only output if particle still alive
		{
			vec3 deltaPos = uni_TimeDelta * var_PVelocity[0];
			vec3 slowDown = 30 * vec3(0.0,-9.81,0.0) * uni_TimeDelta;
	
			out_Type = PTYPE_SIMPLE;
			out_Position = var_Pos[0] + deltaPos;
			out_Rotation = max(var_Rotation[0] - 0.05 * uni_TimeDelta,0.0); // rotation is used for alpha
			out_Scale = max(var_Scale[0] - (var_Scale[0] * 0.50 * uni_TimeDelta),0.0);
			out_Velocity = var_PVelocity[0] + slowDown;
			out_Lifetime = age;
			EmitVertex();
			EndPrimitive();
		}
	}
}