#version 330

layout (points) in;
layout (triangle_strip) out;
layout (max_vertices = 4) out;

uniform mat4 uni_ProjectionMatrix;

out vec2 var_TexCoord;

// The geometry Shader takes the positions and creates a billboard
// consisting of 4 vertices (transformed right into clip space) out of it
void main(void)
{
	//the position from vertex shader - in view space
	vec3 Pos = gl_in[0].gl_Position.xyz;
	
	float scale = 2.0;
	float dx = 0.5 * scale;
	float dy = 0.5 * scale;
	
	//now create the billboard by creating 4 vertices
	//the point is already in view space, so we can think in 2D
	//the original point is the center of the billboard
	
	gl_Position = uni_ProjectionMatrix * 
						vec4( Pos.xy - vec2( - dx, - dy), Pos.z, 1.0);
	var_TexCoord = vec2(-1.0, -1.0);
	EmitVertex();
	
	gl_Position = uni_ProjectionMatrix * 
						vec4( Pos.xy - vec2( + dx, - dy), Pos.z, 1.0);
	var_TexCoord = vec2(1.0, -1.0);
	EmitVertex();
	
	gl_Position = uni_ProjectionMatrix * 
						vec4( Pos.xy - vec2( - dx, + dy), Pos.z, 1.0);
	var_TexCoord = vec2(-1.0, 1.0);
	EmitVertex();
	
	gl_Position = uni_ProjectionMatrix * 
						vec4( Pos.xy - vec2( + dx, + dy), Pos.z, 1.0);
	var_TexCoord = vec2(1.0, 1.0);
	EmitVertex();	
	
    EndPrimitive();
}