#version 330

uniform sampler2D uni_DiffuseMap;

uniform vec3 uni_ParallelLightDirection;  //already in VIEW space!
uniform vec3 uni_ParallelLightDiffuse;
uniform vec3 uni_ParallelLightAmbient;
uniform vec3 uni_ParallelLightSpecular;

in vec2 var_TexCoord;
in vec3 var_Normal;
in vec3 var_EyeVec; //from point to eye

out vec4 frag_Color;
out vec4 frag_LSOcclusion;

const mat4 colorMatrix = mat4(
   0.30, 0.0, 0.0, 0.0, // first column 
   0.0, 0.6, 0.2, 0.0, // second column
   0.3, 0.4, 0.8, 0.0, // third column
   0.0, 0.0, 0.0, 1.0 // fourth column
);

void shadePhong(in vec3 lightAmbientColor,in vec3 lightDiffuseColor,in vec3 lightSpecularColor,
                in vec3 normal,in vec3 lightDirection, in vec3 eyeVector, out vec3 textureColor , out vec3 specular)
{
    //diffuse light = DiffMaterial * DiffLight * Lambert
	float lambert = max(dot(normal,-lightDirection),0.0);
	vec3 diffuse = lambert * lightDiffuseColor;
	
	//ambient light = ambientMaterial * ambientLight
	vec3 ambient = lightAmbientColor;

    //specular light = specular_map * specLight * phong
	float spec = 0.0;

	if(lambert>0.0) //only calc if light shines on this point
	{
		vec3 R = reflect(lightDirection,normal);		
		vec3 E = eyeVector; 

		spec = pow(max(dot(normalize(R),E),0.0),16);
	}
    specular = spec * lightSpecularColor;
    textureColor = ambient + diffuse;
}

void main(void)
{
	//get color from texture
	vec4 texCol = texture(uni_DiffuseMap, var_TexCoord);
	
	//do alpha test
	if(texCol.a < 0.6)
		discard;
	
	//normalize normals
	vec3 normal = normalize(var_Normal);

	vec3 textureLighting;
    vec3 specularLighting;
	shadePhong(uni_ParallelLightAmbient,uni_ParallelLightDiffuse,uni_ParallelLightSpecular,
               normal,uni_ParallelLightDirection,var_EyeVec,textureLighting,specularLighting);
		
    vec4 shadedColor = vec4(textureLighting * texCol.rgb + specularLighting, 1.0);
    
	if(var_EyeVec.y > 200.0f)
		shadedColor = colorMatrix * shadedColor;
	
	frag_Color = shadedColor;
    frag_LSOcclusion = vec4(0,0,0,1.0);
}