#version 330

uniform sampler2D uni_DiffuseMap;
uniform sampler2D uni_SpecularMap;

uniform vec3 uni_ParallelLightDirection;  //already in VIEW space!
uniform vec3 uni_ParallelLightDiffuse;
uniform vec3 uni_ParallelLightAmbient;
uniform vec3 uni_ParallelLightSpecular;
uniform float uni_WaterHeight;

in vec4 var_WorldPosition;
in vec3 var_Normal;
in vec3 var_NormalViewSpace;
in vec3 var_EyeVec; //from point to eye

out vec4 frag_Color;

void shadePhong(in vec3 lightAmbientColor,in vec3 lightDiffuseColor,in vec3 lightSpecularColor,
                in vec3 normal,in vec3 lightDirection, in vec3 eyeVector, out vec3 textureColor , out vec3 specular)
{
    //diffuse light = DiffMaterial * DiffLight * Lambert
	float lambert = max(dot(normal,-lightDirection),0.0);
	vec3 diffuse = lambert * lightDiffuseColor;
	
	//ambient light = ambientMaterial * ambientLight
	vec3 ambient = lightAmbientColor;

    //specular light = specular_map * specLight * phong
	float spec = 0.0;

	if(lambert>0.0) //only calc if light shines on this point
	{
		vec3 R = reflect(lightDirection,normal);		
		vec3 E = eyeVector; 

		spec = pow(max(dot(normalize(R),E),0.0),16);
	}
    specular = spec * lightSpecularColor;
    textureColor = ambient + diffuse;
}

void main(void)
{
   vec3 absNormal = abs(var_Normal);
   vec3 blend_weights = absNormal;
   blend_weights = blend_weights - 0.2679f;
   blend_weights = max(blend_weights, 0);
   
   // force sum to 1.0
   float sum = blend_weights.x + blend_weights.y + blend_weights.z;
   blend_weights = blend_weights.xyz / sum;

   float tex_scale = 0.01f;

   vec2 coord1 = var_WorldPosition.yz * tex_scale;
   vec2 coord2 = var_WorldPosition.zx * tex_scale;
   vec2 coord3 = var_WorldPosition.xy * tex_scale;

   //alpha is directly used for the specular maps weighting
   vec4 col1 = vec4(texture(uni_DiffuseMap, coord1).rgb, texture(uni_SpecularMap, coord1).r); 
   vec4 col2 = vec4(texture(uni_DiffuseMap, coord2).rgb, texture(uni_SpecularMap, coord2).r); 
   vec4 col3 = vec4(texture(uni_DiffuseMap, coord3).rgb, texture(uni_SpecularMap, coord3).r);

   //debug colorize - multiply instead of addition
   //col1 = col1*vec4(1.0,0.0,0.0,1.0); 
   //col2 = col2*vec4(0.0,1.0,0.0,1.0);
   //col3 = col3*vec4(0.0,0.0,1.0,1.0);
   
   vec4 blended_color = col1.rgba * blend_weights.xxxx + 
						col2.rgba * blend_weights.yyyy + 
						col3.rgba * blend_weights.zzzz;
	
    //do some color scaling depending on depth
    const float colMin = 0.1;

    float depthScale = 1.0 - clamp((uni_WaterHeight-var_WorldPosition.y) / uni_WaterHeight,0.0,1.0-colMin);
    depthScale = pow(depthScale,3);

	//now do phong shading
	vec3 shadeNormal = normalize(var_NormalViewSpace);

    vec3 texLighting;
    vec3 specularLighting;
    shadePhong(uni_ParallelLightAmbient,uni_ParallelLightDiffuse,uni_ParallelLightSpecular,
                                shadeNormal,uni_ParallelLightDirection,var_EyeVec,texLighting,specularLighting);
    vec3 shadedColor = texLighting * blended_color.rgb + specularLighting * blended_color.a;
    frag_Color = vec4(shadedColor*depthScale, 1.0);
}