#version 330

uniform sampler2D uni_DiffuseMap;
uniform sampler2D uni_SpecularMap;



in vec4 var_WorldPosition;
in vec3 var_Normal;

in vec3 var_TextureLighting;
in vec3 var_SpecularLighting;

out vec4 frag_Color;
out vec4 frag_LSOcclusion;

void main(void)
{
   vec3 absNormal = abs(var_Normal);
   vec3 blend_weights = absNormal;
   blend_weights = blend_weights - 0.2679f;
   blend_weights = max(blend_weights, 0);
   
   // force sum to 1.0
   float sum = blend_weights.x + blend_weights.y + blend_weights.z;
   blend_weights = blend_weights.xyz / sum;

   float tex_scale = 0.01f;

   vec2 coord1 = var_WorldPosition.yz * tex_scale;
   vec2 coord2 = var_WorldPosition.zx * tex_scale;
   vec2 coord3 = var_WorldPosition.xy * tex_scale;

   //alpha is directly used for the specular maps weighting
   vec4 col1 = vec4(texture(uni_DiffuseMap, coord1).rgb, texture(uni_SpecularMap, coord1).r); 
   vec4 col2 = vec4(texture(uni_DiffuseMap, coord2).rgb, texture(uni_SpecularMap, coord2).r); 
   vec4 col3 = vec4(texture(uni_DiffuseMap, coord3).rgb, texture(uni_SpecularMap, coord3).r);

   //debug colorize - multiply instead of addition
   //col1 = col1*vec4(1.0,0.0,0.0,1.0); 
   //col2 = col2*vec4(0.0,1.0,0.0,1.0);
   //col3 = col3*vec4(0.0,0.0,1.0,1.0);
   
   vec4 blended_color = col1.rgba * blend_weights.xxxx + 
						col2.rgba * blend_weights.yyyy + 
						col3.rgba * blend_weights.zzzz;


    vec3 shadedColor = var_TextureLighting * blended_color.rgb + var_SpecularLighting * blended_color.a;
    frag_Color = vec4(shadedColor, 1.0);
    frag_LSOcclusion = vec4(0,0,0,1.0);
}