#version 330

//need all 3 mats
uniform mat4 uni_ModelViewProjMatrix;
uniform mat4 uni_ModelViewMatrix;
uniform mat4 uni_ModelMatrix;

uniform mat3 uni_NormalMatrix; //matrix to transform normals to VIEW space


uniform vec3 uni_ParallelLightDirection;  //already in VIEW space!
uniform vec3 uni_ParallelLightDiffuse;
uniform vec3 uni_ParallelLightAmbient;
uniform vec3 uni_ParallelLightSpecular;
uniform float uni_WaterHeight;

in vec3 in_Vertex;
in vec3 in_Normal;

out vec4 var_WorldPosition;
out vec3 var_Normal; //unmodified normal - used for triplanar texturing

out vec3 var_TextureLighting;
out vec3 var_SpecularLighting;

void shadePhong(in vec3 lightAmbientColor,in vec3 lightDiffuseColor,in vec3 lightSpecularColor,
                in vec3 normal,in vec3 lightDirection, in vec3 eyeVector, out vec3 textureColor , out vec3 specular)
{
    //diffuse light = DiffMaterial * DiffLight * Lambert
	float lambert = max(dot(normal,-lightDirection),0.0);
	vec3 diffuse = lambert * lightDiffuseColor;
	
	//ambient light = ambientMaterial * ambientLight
	vec3 ambient = lightAmbientColor;

    //specular light = specular_map * specLight * phong
	float spec = 0.0;

	if(lambert>0.0) //only calc if light shines on this point
	{
		vec3 R = reflect(lightDirection,normal);		
		vec3 E = eyeVector; 

		spec = pow(max(dot(normalize(R),E),0.0),16);
	}
    specular = spec * lightSpecularColor;
    textureColor = ambient + diffuse;
}

void main(void)
{
	vec4 vtx = vec4(in_Vertex,1);

	//pos in world space
	var_WorldPosition =  uni_ModelMatrix * vtx;
	//pos in view space
	vec4 viewPos = uni_ModelViewMatrix * vtx;

	vec3 eyeVec = normalize(-viewPos.xyz);

	vec3 normalViewSpace = normalize(uni_NormalMatrix * in_Normal);
	
	//do some color scaling depending on depth
    const float colMin = 0.1;

    float depthScale = 1.0 - clamp((uni_WaterHeight-var_WorldPosition.y) / uni_WaterHeight,0.0,1.0-colMin);
    depthScale = pow(depthScale,3);
	
	shadePhong(uni_ParallelLightAmbient,uni_ParallelLightDiffuse,uni_ParallelLightSpecular,
               normalViewSpace,uni_ParallelLightDirection,eyeVec,var_TextureLighting,var_SpecularLighting);
			   
	var_TextureLighting = depthScale*var_TextureLighting;
	var_SpecularLighting = depthScale*var_SpecularLighting;
	
	var_Normal = in_Normal;

	//final vtx position
	gl_Position = uni_ModelViewProjMatrix * vtx;
}