#version 330

const float pi = 3.14159;

uniform float uni_WaterHeight;
uniform float uni_Time;
uniform float uni_Amplitude[4];
uniform float uni_Frequency[4];
uniform float uni_Phase[4];
uniform vec2 uni_Direction[4];

in vec3 in_Vertex;

out vec3 var_NormalObjSpace; //normal in object space
//out vec3 var_Normal; //world normal
//out vec3 var_EyeVec; //from point to eye

float wave(int i, float x, float y) {
    float theta = dot(uni_Direction[i], vec2(x, y));
    return uni_Amplitude[i] * sin(theta * uni_Frequency[i] + uni_Time * uni_Phase[i]);
}

float waveHeight(float x, float y) {
    float height = 0.0;
    for (int i = 0; i < 4; ++i)
        height += wave(i, x, y);
    return height;
}

float dWavedx(int i, float x, float y) {
    float theta = dot(uni_Direction[i], vec2(x, y));
    float A = uni_Amplitude[i] * uni_Direction[i].x * uni_Frequency[i];
    return A * cos(theta * uni_Frequency[i] + uni_Time * uni_Phase[i]);
}

float dWavedy(int i, float x, float y) {
    float theta = dot(uni_Direction[i], vec2(x, y));
    float A = uni_Amplitude[i] * uni_Direction[i].y * uni_Frequency[i];
    return A * cos(theta * uni_Frequency[i] + uni_Time * uni_Phase[i]);
}

vec3 waveNormal(float x, float y) {
    float dx = 0.0;
    float dy = 0.0;
    for (int i = 0; i < 4; ++i) {
        dx += dWavedx(i, x, y);
        dy += dWavedy(i, x, y);
    }
    vec3 n = vec3(-dx, 1.0, -dy);
    return n;
}

void main(void)
{
	vec4 vtx = vec4(in_Vertex,1);

	vtx.y = uni_WaterHeight + waveHeight(vtx.x, vtx.z);
	var_NormalObjSpace = normalize(waveNormal(vtx.x,vtx.z));
	//var_Normal = normalize(uni_NormalMatrix * waveNormal(vtx.x, vtx.z));
	
	//pos in view space
	//vec4 viewPos = uni_ModelViewMatrix * vtx;
	//var_EyeVec = -normalize(viewPos.xyz);
	
	//needs unmodified vertex, all transformations occur in GS
	gl_Position = vtx;
	//gl_Position =  uni_ModelViewProjMatrix * vtx;
}