#version 330

in vec3 var_NormalViewSpace;
in vec3 var_Normal;
in vec3 var_EyeVec; //from point to eye
in vec3 var_EyeVecViewSpace;
in float var_WaterDepth;

out vec4 frag_Color;
out vec4 frag_LSOcclusion;

uniform samplerCube uni_CubeMap;
uniform vec3 uni_ParallelLightDirection;
uniform float uni_WaterHeight;
uniform vec3 uni_CameraPos;

void main(void)
{
	//vec3 r = normalize(reflect(normalize(var_EyeVec), vec3(0,1,0)));
	vec3 r = vec3(0.0,0.0,0.0);
	
	//reflections only if over water
	if(uni_CameraPos.y > 200.0)
		r = normalize(reflect(normalize(var_EyeVec), normalize(var_Normal)));

	//r = uni_NormalMatrix * r;
	vec3 color = texture(uni_CubeMap, r).rgb;
	
	
	//at coast, make a bit darker and more transparent
	float beachFactor = 1-smoothstep(0.0,5.0,var_WaterDepth);
	float coastFactor = 1-smoothstep(0.0,40.0,var_WaterDepth);
	
	float shallowFactor = 1-smoothstep(0.0,200.0,var_WaterDepth);
	float deepSeaFactor = 1-smoothstep(175.0,uni_WaterHeight,var_WaterDepth);
	
	//make unperfect reflection
	color = mix(color,vec3(0.1,0.3,0.4),0.1+shallowFactor*0.9);
	
	color = mix(color,vec3(0.5,0.65,0.7),coastFactor);
	color = mix(color,vec3(1.0,1.0,1.0),beachFactor);
	
	color = mix(color,color*0.7,1-deepSeaFactor);

	float eyedistfactor = 1-smoothstep(0.0,500.0,length(var_EyeVec));
	float alphaEyeDist = mix(0.95,0.4,eyedistfactor);
	float alphaCoastDist= mix(0.95,0.2,pow(coastFactor,4.0));
	
	float alpha = min(alphaEyeDist,alphaCoastDist);

	if(var_WaterDepth>uni_WaterHeight)
		alpha=1.0;
	
	//shade a bit!
	vec3 normal = normalize(var_NormalViewSpace);
	float lambert = max(dot(normal,-uni_ParallelLightDirection),0.0);
	vec3 sunR = reflect(uni_ParallelLightDirection,normal);
	vec3 E = normalize(- var_EyeVecViewSpace);
	float spec = pow(max(dot(normalize(sunR),E),0.0),64);
	color = color * lambert + color * 0.2 + spec;
	
	//TODO: make alpha dependent on view angle
	frag_Color = vec4(color,alpha);
	//frag_Color = vec4(color,1.0);
	
	//frag_Color = vec4(r,1.0);
	//frag_Color = vec4(var_Normal,1.0);
    frag_LSOcclusion = vec4(vec3(spec*0.1),1.0);
}