#version 330

const float pi = 3.14159;

uniform mat4 uni_ModelViewProjMatrix;
uniform mat4 uni_ModelViewMatrix;
uniform mat3 uni_NormalMatrix;

uniform float uni_WaterHeight;
uniform float uni_Time;
uniform float uni_Amplitude[4];
uniform float uni_Frequency[4];
uniform float uni_Phase[4];
uniform vec2 uni_Direction[4];

uniform vec3 uni_CameraPos; //worldspace

in vec3 in_Vertex;

out vec3 var_NormalViewSpace;
out vec3 var_Normal; //world normal
out vec3 var_EyeVecViewSpace;
out vec3 var_EyeVec; //from point to eye
out float var_WaterDepth; //depth of water here

float wave(int i, float x, float y) {
    float theta = dot(uni_Direction[i], vec2(x, y));
    return uni_Amplitude[i] * sin(theta * uni_Frequency[i] + uni_Time * uni_Phase[i]);
}

float waveHeight(float x, float y) {
    float height = 0.0;
    for (int i = 0; i < 4; ++i)
        height += wave(i, x, y);
    return height;
}

float dWavedx(int i, float x, float y) {
    float theta = dot(uni_Direction[i], vec2(x, y));
    float A = uni_Amplitude[i] * uni_Direction[i].x * uni_Frequency[i];
    return A * cos(theta * uni_Frequency[i] + uni_Time * uni_Phase[i]);
}

float dWavedy(int i, float x, float y) {
    float theta = dot(uni_Direction[i], vec2(x, y));
    float A = uni_Amplitude[i] * uni_Direction[i].y * uni_Frequency[i];
    return A * cos(theta * uni_Frequency[i] + uni_Time * uni_Phase[i]);
}

vec3 waveNormal(float x, float y) {
    float dx = 0.0;
    float dy = 0.0;
    for (int i = 0; i < 4; ++i) {
        dx += dWavedx(i, x, y);
        dy += dWavedy(i, x, y);
    }
    vec3 n = vec3(-dx, 1.0, -dy);
    return n;
}

void main(void)
{
	vec4 vtx = vec4(in_Vertex,1);
	
	//y stores heightmap height
	var_WaterDepth = uni_WaterHeight - vtx.y;
	vtx.y = uni_WaterHeight + waveHeight(vtx.x, vtx.z);
	//also looks good when flat!
	//vtx.y = uni_WaterHeight;
	
	//normal NEEDS to be in world space
	//var_Normal = vec3(0,1,0);
	var_Normal = waveNormal(vtx.x,vtx.z);
	var_NormalViewSpace = uni_NormalMatrix * var_Normal;
	
	//eyevec in world space
	var_EyeVec = vtx.xyz - uni_CameraPos;
	var_EyeVecViewSpace = (uni_ModelViewMatrix * vtx).xyz;
	
	gl_Position =  uni_ModelViewProjMatrix * vtx;
}